#
# UIM restful maintenance mode 
# - add computer system to EXISTING schedule based on an input file with the requested server names
# or
# - create calendar and add the defined servers directly to that just created calendar
#   - we will do a lookup in cm_computer_system to find the related cs_id (query to obtain cs_id: select name,cs_id from CM_COMPUTER_SYSTEM where (nimbus_type = '1' or nimbus_type = '2'))
#  - we will read the input file with a line per server that you want to put in maintenance
#  - when we find the server in the list with a related cs_id we will add it in the request list
#  - at the end we will do 1 POST request to create the calendar , save the calendar id and 1 post for the add server with teh saved calendar id
# or
# - with option -lc"y" we will give a list of existing calendars so that you can obtain the needed calendar id
#
# Version 2.6 03/04/2019 - Luc Christiaens 
# (based on rest sample from Armin Gross)
#
# *** Pre-Req: ***
# - you need:  nimsoft_generic.pm and nimsoft_generic.dat
# -   when you run in perl mode, copy nimsoft_generic.pm in your /perl/lib directory (else it's compiled into the exe) or in the directory with the .pl source
# -   copy nimsoft_generic.dat in the same directory as the perl module (.pl or .exe)
# - Edit nimsoft_generic.dat and change all variables, they will be common between all my UIM/Nimsoft Perl 
#
# Updates:
# - 01/08/2015: initial version
# - 10/08/2015: add option -lc to list defined calendars to obtain calendar id needed to add computers to existing calendar
# -           : add option -if to define path\file with list of servers to be put/delete in maintenance
# -           : add option -ci to define the calendar id of an existing calendar to be used (you can use -l to have a list of existing calendars)
# - 02/09/2015: add option -ac to define the action: a (add) or r (remove) or c (calendar+server) (c is the default)
# - 24/09/2015: add option -ih to define a list of comma separated hostnames to put/delete in maintenance (if -h used -i is not taken)
#             : add option -lm to list machines in maintenance
#             : add use for compile
# - 25/09/15  : add sql_db as parameter      
#             : add check for availibility of nimsoft_generic.dat (in nimsoft_generic.pm)
#             : add cd to run from the directory where module is located (nimsoft_generic.dat must be in that directory)
#             : add option -db"y" to have more debug info
# - 09/11/15  : change date format for start_time and end_time (from 100 to 120) 
#             : format output of -lc and -lm
# - 07/12/15  : nimsoft_generic.dat will only use passwords that are crypted via nimsoft_crypt.pl
# - 15/02/16  : sql user "trusted" and password blank will trigger a trusted sql connection
# - 14/04/16  : adapted usage to show more clear the need of "y" in -l and -m
# - 06/06/16  : add -ls.  You enter a hostname and we will return rc=0 is server is IN maintenance, else rc=1 (example -ih"servername")
# - 22/06/16  : adapt -lc, -ls, -lm for mysql
# - 24/06/16  : add also nimbus_type: 0 (servers without entry in hub?)
# - 19/08/16  : add -le: list empty calendars (y,n) default: n
#             : add -lx: list expired calendars (y,n) deafault: n (where the cal end_time < now)
# - 02/09/16  : changed all input switches to 2 characters (to prepare to add for all calendar definition options)
#             : -cs: define calendar duration in seconds
#             : -cu: calendar until in format yyyy-mm-dd hh:mm:ss
#             : -cf: calendar from  in format yyyy-mm-dd hh:mm:ss
#             : -ct: calendar title (if not defined will use list of servers) 
#             : -dx: delete expired calendars 
# - 02/11/16  : adapted sql query for -le to list only expired calendars (and not list expired servers)
#             : move from XML::Simple to XML::LibXML
# - 08/11/16  : -ig: input group. You can now use the last qualifier of an USM group as selection, all servers in this group will be defined in maint
#             : add the possibility to select the servers with generic regex fields:
#                -hi: hub include, -he: hub exclude
#                -ho: hub origin include, -hx: hub origin exclude
#                -ti: robot include, -te: robot exclude
#                -u1: user1 field robot include, -u2: user2 field robot include
# - 09/11/16  :  -ni: device name include
#                -ne: device name exclude 
#                -sim: y/n for server selection the default is y (for all other options: n)
# - 16/11/16  - add parameter checking for server selection
#             - when selecting a group you can now use -ni and -ne to include / exclude servers via regex
#             - verify for action: a,r, c if Rest services are responding, if not "exit 404";
# - 28/11/16  - adapt -le for mysql 
#             - no mysql yet for -lx and -dx
# - 30/11/16  - add -de (delete empty calendars)
#             - adapt -lx for mysql
# - 15/12/16  - if -ac"r" is used we need always a calendar_id via -ci
#             - simulate=y by default for all options  
# - 21/12/16  - add hub and origin in print of selected devices
# - 22/12/16  - for the list options we don't need to check if Rest services are working
# - 01/02/17  - add uim_https option in nimsoft_generic.dat and add option to avoid certificate errors when using https
# - 18/04/17  - correct query to have a list of devices from a group, even if the devices are not a robot 
# - 28/04/17  - add with(nolock)
# - 09/03/18  - -ls option had the with(nolock) incorrect
# - 16/03/18  - add -dm"server" (regex) to remove servers from maintenance: specify 0 seconds and the server will be removed from maint (even if the server is located in multiple calendars)
#             - if cm_computer_system contains multiple entries for the same server, we will put all corresponding cs_ids in maintenance
#             - translate all servernames to uppercase so that multiple occurrences are grouped and can have multiple cs_ids 
# - 30/03/18  - to be clear:
#                 - -h"server1,server2" are non-regex name(s) to add a server in maint
#                 - -dm"server1|server2" is a regex string  to remove a server from maint
# - 03/04/19  - add option -ir (include regex) that makes it possible to create a regex device against cm_computer_system
#               (because -ni, that is also regex, is working against cm_nimbus_robot)
#
# Examples:
#  - 1 - list existing calendars: perl nimsoft_rest_maintenance.pl -lc"y"
#  - 2 - list servers currently in maintenance: perl nimsoft_rest_maintenance.pl -lm"y"
#  - 3 - set a list of servers (in a file) in maintenance based on existing calendar id id: 3 and an existing list of servers (1 server per line): perl nimsoft_rest_maintenance.pl -if"c:\temp\rest_maint_computers.txt" -ci"3" -ac"a"
#  - 4 - set a comma separated list of servers in maintenance for 3600 seconds: perl nimsoft_rest_maintenance.pl -ih"server1,server2" -ac"c" -cs"3600" -ct"cal test"
#  - 5 - remove a server from maintenance (with the calendar-id): perl nimsoft_rest_maintenance.pl -ih"server2" -ci"3" -ac"r"
#  - 6 - set all servers from USM group bgbulab in maintenance for xx seconds, excluding possible hub robots: perl nimsoft_rest_maintenance.pl -ig"bgbulab" -cs"36000" 
#  - 7 - remove a server from maintenance: perl nimsoft_rest_maintenance.pl -dm"server1|server2" 
#  - 8 - set all devices matching fake-server in maintenance for 3600 seconds: perl nimsoft_rest_maintenance.pl -ir"fake-server" -ct"fake-server-calendar" -cs"3600"
#
# TODO:
#
#
# ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
# - This program is only an example program that can be used on your own risk
# ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
#


#!/usr/bin/perl
#use strict;
#use warnings;
# some packages we use
use XML::LibXML;
use Data::Dumper;
use LWP::UserAgent;
use HTTP::Request;
use DBI;
use DBD::ODBC;
use Getopt::Std;
# crypt
use Crypt::RC4;
use MIME::Base64;
# time
use Time::Piece;
use Time::Seconds qw/ ONE_DAY /;
# - crypt key
$crkey="ab_def%hij&5";
$pgm="Nimsoft_Rest_Maintenance (v2.6)";

# --- get parameters
&getparams;

# --- change directory to directory where perl source is located to find our nimsoft_generic.dat file
use Cwd 'chdir';
use Cwd;
use File::Basename;
chdir dirname $0;
$curdir=cwd();

# --- variables
$hubcount=0;

# --- UIM generic package
use nimsoft_generic;

# --- prepare variables
&prep_var;

# --- recalculate date and time
&calcdate;

#
# --- check if WASP & REST are available (and come only back if we have a response)
#
if (($action eq 'a' || $action eq 'c' || $action eq 'r') && ($list_calendar ne 'y' && $list_maint ne 'y' && $list_empty ne 'y' && $list_expired ne 'y'))
 {
  while ($hubcount eq '0')
   {
     print "Rest_Check - Start check if REST services are up and running\n";
     &check_rest;
# if we want to wait until ok, remove exit in check and set a switch to test
   }
 }

#
# step_lc - do we have parameter -lc"y" to create a list of existing calendars?
#

if ($list_calendar eq 'y')
{
# --- connect DB ---
&connect_db;

# --- prepare sql query (nimbus_type: 0: fake entry 1=robot 2=hub)
if ($sqltype eq 'my')
  {
     $sql = "select SCHEDULE_ID,SCHEDULE_NAME, DESCRIPTION, TIME_FORMAT(START_TIME, '%H:%i:%s') as 'Start Time', Duration from MAINTENANCE_SCHEDULE";
  }
if ($sqltype eq 'ms')
  { 
     $sql = "select SCHEDULE_ID,SCHEDULE_NAME, DESCRIPTION, CONVERT(VARCHAR(50), START_TIME,120) as 'Start Time', Duration from MAINTENANCE_SCHEDULE with(nolock)";
  }

# --- execute query
$sth = $dbh->prepare($sql);
$sth->execute or die "SQL Error: $DBI::errstr\n";

# --- loop in output
while (@row = $sth->fetchrow_array) 
 {
# - decode line
  ($db_sch_id, $db_sch_name, $db_sch_desc,$db_sch_start,$db_sch_duration) = @row;
  $counts++;
# --- add found computer in table for later direct reference
   print "$counts - calendar_id: $db_sch_id Name: $db_sch_name Description: $db_sch_desc \n";
   print "    Startdate: $db_sch_start Duration: $db_sch_duration minutes \n";

 }

print "\n";
print "step 0: We found $counts calendars defined\n";

# --- disconnect db
$dbh->disconnect();

# --- here is the end for option -lc"y"
exit;
}

#
# step_lm - do we have parameter -lm"y" to create a list of hosts in maintenance?
#

if ($list_maint eq 'y')
{
# --- connect DB ---
&connect_db;

# --- prepare sql query (nimbus_type: 0=entry without robot 1=robot 2=hub)
# - to list removed since yesterday:  WHERE mw.END_TIME >=DATEADD(DD, -1,GETDATE())
# - to list added since yesterday: WHERE mw.START_TIME <=DATEADD(DD, -1,GETDATE())
if ($sqltype eq 'my')
  {
    $sql = "select distinct cs.name, cs.ip, cs.origin,ms.schedule_id, ms.SCHEDULE_NAME, ms.DESCRIPTION, DATE_FORMAT(mw.START_TIME, '%d-%m-%Y') as 'Start Date', TIME_FORMAT(ms.START_TIME, '%H:%i:%s') as 'Start Time', DATE_FORMAT(mw.END_TIME, '%d-%m-%Y') as 'End Date', TIME_FORMAT(mw.END_TIME, '%H:%i:%s') as 'End Time'  from CM_COMPUTER_SYSTEM cs INNER JOIN MAINTENANCE_SCHEDULE_MEMBERS msm  ON cs.cs_id=msm.CS_ID  INNER JOIN MAINTENANCE_WINDOW mw  ON msm.SCHEDULE_ID=mw.SCHEDULE_ID  INNER JOIN MAINTENANCE_SCHEDULE ms  ON msm.SCHEDULE_ID=ms.SCHEDULE_ID  WHERE mw.END_TIME >=GETDATE()  group by name,ip,origin,ms.schedule_id,ms.SCHEDULE_NAME,ms.DESCRIPTION";
  }
if ($sqltype eq 'ms')
  {
    $sql = "select distinct cs.name, cs.ip, cs.origin,ms.schedule_id, ms.SCHEDULE_NAME, ms.DESCRIPTION, max(CONVERT(VARCHAR(50), ms.START_TIME,111)) as 'Start Date', max(CONVERT(VARCHAR(50), ms.START_TIME,108)) as 'Start Time', max(CONVERT(VARCHAR(50), mw.END_TIME, 111)) as 'End Date', max(CONVERT(VARCHAR(50), mw.END_TIME, 108)) as 'End Time' from CM_COMPUTER_SYSTEM cs with(nolock) INNER JOIN MAINTENANCE_SCHEDULE_MEMBERS msm with(nolock)  ON cs.cs_id=msm.CS_ID  INNER JOIN MAINTENANCE_WINDOW mw with(nolock)  ON msm.SCHEDULE_ID=mw.SCHEDULE_ID  INNER JOIN MAINTENANCE_SCHEDULE ms with(nolock)  ON msm.SCHEDULE_ID=ms.SCHEDULE_ID  WHERE mw.END_TIME >=GETDATE()  group by name,ip,origin,ms.schedule_id,ms.SCHEDULE_NAME,ms.DESCRIPTION";
  } 

# --- execute query
$sth = $dbh->prepare($sql);
$sth->execute or die "SQL Error: $DBI::errstr\n";

# --- loop in output
while (@row = $sth->fetchrow_array) 
 {
# - decode line (its normal that some variables are not used like they are common between several perls)
  ($m_host,$m_ip,$m_origin,$m_sch_id,$m_schedule,$m_description, $m_startdate, $m_starttime,$m_enddate, $m_endtime,) = @row;
  $countm++;
  print "$countm - calendar_id: $m_sch_id Name: $m_schedule \n";
  print "    Start: $m_startdate $m_starttime End: $m_enddate $m_endtime Host: $m_host\n";

 }

print "\n";
print "step 0: We found $countm hosts in maintenance\n";

# --- disconnect db
$dbh->disconnect();

# --- here is the end for option -lm"y"
exit;
}

#
# step_ls - do we have parameter -ls"servername" that we need to check if it's in maintenance?
#

if ($ismachine ne '')
{
# --- connect DB ---
&connect_db;

# --- prepare sql query (nimbus_type: 0= 1=robot 2=hub)
# - to list removed since yesterday:  WHERE mw.END_TIME >=DATEADD(DD, -1,GETDATE())
# - to list added since yesterday: WHERE mw.START_TIME >=DATEADD(DD, -1,GETDATE())
if ($sqltype eq 'my')
  {
     $sql = "select distinct cs.name, cs.ip, cs.origin,ms.schedule_id, ms.SCHEDULE_NAME, ms.DESCRIPTION, TIME_FORMAT(ms.START_TIME, '%H:%i:%s') as 'Start Time',
TIME_FORMAT(mw.END_TIME, '%H:%i:%s') as 'End Time' from CM_COMPUTER_SYSTEM cs INNER JOIN MAINTENANCE_SCHEDULE_MEMBERS msm  ON cs.cs_id=msm.CS_ID  INNER JOIN MAINTENANCE_WINDOW mw  ON msm.SCHEDULE_ID=mw.SCHEDULE_ID  INNER JOIN MAINTENANCE_SCHEDULE ms  ON msm.SCHEDULE_ID=ms.SCHEDULE_ID  WHERE mw.END_TIME >=GETDATE()  group by name,ip,origin,ms.schedule_id,ms.SCHEDULE_NAME,ms.DESCRIPTION";
  }
if ($sqltype eq 'ms')
  {
    $sql = "select distinct cs.name, cs.ip, cs.origin,ms.schedule_id, ms.SCHEDULE_NAME, ms.DESCRIPTION, max(CONVERT(VARCHAR(50), ms.START_TIME,120)) as 'Start Time', max(CONVERT(VARCHAR(50), mw.END_TIME, 120)) as 'End Time' from CM_COMPUTER_SYSTEM  cs with(nolock) INNER JOIN MAINTENANCE_SCHEDULE_MEMBERS msm with(nolock) ON cs.cs_id=msm.CS_ID  INNER JOIN MAINTENANCE_WINDOW mw with(nolock) ON msm.SCHEDULE_ID=mw.SCHEDULE_ID  INNER JOIN MAINTENANCE_SCHEDULE ms with(nolock) ON msm.SCHEDULE_ID=ms.SCHEDULE_ID  WHERE mw.END_TIME >=GETDATE()  group by name,ip,origin,ms.schedule_id,ms.SCHEDULE_NAME,ms.DESCRIPTION";
  }

# --- execute query
$sth = $dbh->prepare($sql);
$sth->execute or die "SQL Error: $DBI::errstr\n";

# --- loop in output
while (@row = $sth->fetchrow_array) 
 {
# - decode line (its normal that some variables are not used like they are common between several perls)
  ($m_host,$m_ip,$m_origin,$m_sch_id,$m_schedule,$m_description, $m_start,$m_end,) = @row;
  if ($m_host =~ /$ismachine/i)
   { 
     $countm++;
     print "$countm - calendar_id: $m_sch_id Name: $m_schedule \n";
     print "    Start: $m_start End: $m_end Host: $m_host\n";
   }
 }

print "\n";
print "step 0: We found $countm hosts in maintenance\n";

# --- disconnect db
$dbh->disconnect();

# --- here is the end for option -ls"y"
if ($countm > 0)
 {
   exit;
 }
else
 {
   exit 1;
 }
}

#
# step_dm_1 - do we have parameter -dm"servername" that we need to remove from maintenance?
#

if ($delete_maint ne 'n')
{
# --- connect DB ---
&connect_db;

# --- prepare sql query (nimbus_type: 0= 1=robot 2=hub)
# - to list removed since yesterday:  WHERE mw.END_TIME >=DATEADD(DD, -1,GETDATE())
# - to list added since yesterday: WHERE mw.START_TIME >=DATEADD(DD, -1,GETDATE())
if ($sqltype eq 'my')
  {
     $sql = "select distinct cs.name, cs.ip, cs.origin,ms.schedule_id, ms.SCHEDULE_NAME, ms.DESCRIPTION, TIME_FORMAT(ms.START_TIME, '%H:%i:%s') as 'Start Time',
TIME_FORMAT(mw.END_TIME, '%H:%i:%s') as 'End Time' from CM_COMPUTER_SYSTEM cs INNER JOIN MAINTENANCE_SCHEDULE_MEMBERS msm  ON cs.cs_id=msm.CS_ID  INNER JOIN MAINTENANCE_WINDOW mw  ON msm.SCHEDULE_ID=mw.SCHEDULE_ID  INNER JOIN MAINTENANCE_SCHEDULE ms  ON msm.SCHEDULE_ID=ms.SCHEDULE_ID  WHERE mw.END_TIME >=GETDATE()  group by name,ip,origin,ms.schedule_id,ms.SCHEDULE_NAME,ms.DESCRIPTION";
  }
if ($sqltype eq 'ms')
  {
    $sql = "select distinct cs.name, cs.ip, cs.origin,ms.schedule_id, ms.SCHEDULE_NAME, ms.DESCRIPTION, max(CONVERT(VARCHAR(50), ms.START_TIME,120)) as 'Start Time', max(CONVERT(VARCHAR(50), mw.END_TIME, 120)) as 'End Time',msm.CS_ID from CM_COMPUTER_SYSTEM  cs with(nolock) INNER JOIN MAINTENANCE_SCHEDULE_MEMBERS msm with(nolock) ON cs.cs_id=msm.CS_ID  INNER JOIN MAINTENANCE_WINDOW mw with(nolock) ON msm.SCHEDULE_ID=mw.SCHEDULE_ID  INNER JOIN MAINTENANCE_SCHEDULE ms with(nolock) ON msm.SCHEDULE_ID=ms.SCHEDULE_ID  WHERE mw.END_TIME >=GETDATE()  group by name,ip,origin,ms.schedule_id,ms.SCHEDULE_NAME,ms.DESCRIPTION,msm.CS_ID";
  }

# --- execute query
$sth = $dbh->prepare($sql);
$sth->execute or die "SQL Error: $DBI::errstr\n";

# --- loop in output
while (@row = $sth->fetchrow_array) 
 {
# - decode line (its normal that some variables are not used like they are common between several perls)
  ($m_host,$m_ip,$m_origin,$m_sch_id,$m_schedule,$m_description, $m_start,$m_end,$m_cs_id,) = @row;
  if ($m_host =~ /$delete_maint/i)
   { 
     $countm++;
     print "step_dm_2a: Remove $m_host sched_id: $m_sch_id cs_id: $m_cs_id\n";
#     print "$countm - calendar_id: $m_sch_id Name: $m_schedule \n";
#     print "    Start: $m_start End: $m_end Host: $m_host cs_id: $m_cs_id\n";


#
# step_dm_2: execute restful command to remove server from calendar
#
   if ($simulate eq 'n')
    {

# - Set servername
      $ServerAndPort = $Server_With_Rest_Interface . ":" . $RestServicePort;

# - Define the query URL for UIM RESTful interface
      $baseURL = "$use_https://" . $ServerAndPort . "/rest/maintenance_mode/$UIM_Domain/$UIM_Hub/$UIM_Robot/remove_computer_systems_from_schedule/$m_sch_id";
      if ($debug eq 'y') { print "..  Post action: remove_computer_systems_from_schedule: $action_text url: $baseURL\n";}
 
# - Create LWP Object
      $RestConnection = LWP::UserAgent->new();

# - Create connection
      $realm = "NimJAAS";

      if ($debug eq 'y') { print "..  serverport: $ServerAndPort realm: $realm user: $username pass: $password\n"; }

      $RestConnection->credentials($ServerAndPort, $realm, $username, $password);
# $response=$RestConnection->credentials($ServerAndPort, $realm, $username=>$password);
# print "step4: initial rest connection: $response\n";

# - to avoid https certificate errors
      $RestConnection->ssl_opts( verify_hostname => 0 ,SSL_verify_mode => 0x00);

# - Create POST data
      $post_data = "{ \"cs\":\[$m_cs_id\] }";
      if ($debug eq 'y') {  print "..  We will POST: $post_data \n"; }

# print "step 4a: We will post the remove_computer_systems_from_schedule request to server: $Server_With_Rest_Interface\n";
      print "step_dm_2b: We will post the remove_computer request to server: $ServerAndPort\n";

      $req = HTTP::Request->new(POST => $baseURL);
      $req->content_type('application/json');
      $req->content($post_data);

# - Execute POST        
      $resp = $RestConnection->request($req);

# - Decode response
      if ($resp->is_success) 
       {
         $message = $resp->decoded_content;
         if ($message eq '')
           {
             $message="ok";
             $rc=0;
           } 
         print "step_dm_2c: Received remove_computer reply: $message\n";
       }
     else 
      {
         $rc=$resp->code;
         print "step_dm_2d: HTTP POST error code: ", $resp->code, "\n";
         print "step_dm_2d: HTTP POST error message: ", $resp->message, "\n";
      }
    }
   }
 }

print "\n";
print "step_dm_2e: We found $countm hosts to remove from maintenance\n";
if ($simulate eq 'y')
 {
   print "step_dm_2e: *** Stop execution due to simulation run (-sim\"y\") *** \n";
   exit;
 }

# --- disconnect db
$dbh->disconnect();

# --- here is the end for option -dm"server"
if ($countm > 0)
 {
   exit;
 }
else
 {
   exit 1;
 }
}


#
# step_le - do we have parameter -le"y"? (list empty calendars)
#

if ($list_empty eq 'y')
{
# --- connect DB ---
&connect_db;

# --- prepare sql query (nimbus_type: 0= 1=robot 2=hub)
if ($sqltype eq 'my')
  {
     $sql = "SELECT ms.Schedule_id, Schedule_name, DATE_FORMAT(ms.START_TIME, \'%d-%m-%Y\') as \'Start Date\',TIME_FORMAT(ms.START_TIME, \'%H:%i:%s\') as \'Start Time\', Duration, Description from MAINTENANCE_SCHEDULE ms left join MAINTENANCE_SCHEDULE_MEMBERS mm on mm.SCHEDULE_ID = ms.SCHEDULE_ID where mm.CS_ID IS NULL group by ms.schedule_id,ms.schedule_name,ms.DURATION,ms.description";
  }
if ($sqltype eq 'ms')
  {
    $sql = "SELECT ms.Schedule_id, Schedule_name, max(CONVERT(VARCHAR(50), START_TIME,111)) as \'StartDate\', max(CONVERT(VARCHAR(50), START_TIME,108)) as \'StartTime\', Duration, Description from MAINTENANCE_SCHEDULE ms with(nolock) left join MAINTENANCE_SCHEDULE_MEMBERS mm with(nolock) on mm.SCHEDULE_ID = ms.SCHEDULE_ID where mm.CS_ID IS NULL group by ms.schedule_id,ms.schedule_name,ms.DURATION,ms.description";
  }

# --- execute query
$sth = $dbh->prepare($sql);
$sth->execute or die "SQL Error: $DBI::errstr\n";

# --- loop in output
while (@row = $sth->fetchrow_array) 
 {
# - decode line (its normal that some variables are not used like they are common between several perls)
  ($m_calid,$m_name,$m_startdate, $m_starttime,$m_duration,$m_description) = @row;
     $countm++;
     print "$countm - Calendar_id: $m_calid Name: $m_name \n";
     print "    Start: $m_startdate $m_starttime Duration: $m_duration Description: $m_description\n";
 }

print "\n";
print "step 0: We found $countm empty calendars\n";

# --- disconnect db
$dbh->disconnect();

exit;
}

#
# step_de - do we have parameter -de"y"? (delete empty calendars)
#

if ($delete_empty eq 'y')
{
# --- connect DB ---
&connect_db;

# --- prepare sql query (nimbus_type: 0= 1=robot 2=hub)
if ($sqltype eq 'my')
  {
     $sql = "";
     print "This function/option is not (yet) available for MySQL\n";
     exit 3;
  }
if ($sqltype eq 'ms')
  {
    $sql = "DECLARE \@message varchar(100)\;DECLARE \@cs AS NVARCHAR(500)\;DECLARE devicecursor CURSOR for SELECT ms.SCHEDULE_ID from MAINTENANCE_SCHEDULE ms with(nolock) left join MAINTENANCE_SCHEDULE_MEMBERS msm with(nolock) on msm.SCHEDULE_ID = ms.SCHEDULE_ID where msm.CS_ID IS NULL\; open devicecursor fetch next from devicecursor into \@cs while \@\@FETCH_STATUS = 0 BEGIN delete from MAINTENANCE_SCHEDULE where MAINTENANCE_SCHEDULE.SCHEDULE_ID = \@cs; delete from MAINTENANCE_WINDOW where MAINTENANCE_WINDOW.SCHEDULE_ID = \@cs; FETCH NEXT FROM devicecursor into \@cs END CLOSE devicecursor; DEALLOCATE devicecursor;";
  }

# --- execute query
$sth = $dbh->prepare($sql);
$sth->execute or die "SQL Error: $DBI::errstr\n";
$rows=$sth->rows;
if ($rows eq '1')
 {
   print "Deleted empty calendars from maintenance\n";
 } 

# --- disconnect db
$dbh->disconnect();

# print "Deleted empty calendars from maintenance (rows: $rows)\n";


exit;
}

#
# step_cx - do we have parameter -lx"y"? (list expired calendars)
#

if ($list_expired eq 'y')
{
# --- connect DB ---
&connect_db;

# --- prepare sql query (nimbus_type: 0= 1=robot 2=hub)
if ($sqltype eq 'my')
  {
    $sql = "select ms.schedule_id, ms.SCHEDULE_NAME, ms.DESCRIPTION, DATE_FORMAT(ms.START_TIME, \'%d-%m-%Y\') as \'Start Date\',TIME_FORMAT(ms.START_TIME, \'%H:%i:%s\') as \'Start Time\', DATE_FORMAT(mw.END_TIME, \'%d-%m-%Y\') as \'End Date\',TIME_FORMAT(mw.END_TIME, \'%H:%i:%s\') as \'End Time\' from CM_COMPUTER_SYSTEM cs INNER JOIN MAINTENANCE_SCHEDULE_MEMBERS msm  ON cs.cs_id=msm.CS_ID INNER JOIN MAINTENANCE_WINDOW mw  ON msm.SCHEDULE_ID=mw.SCHEDULE_ID  INNER JOIN MAINTENANCE_SCHEDULE ms  ON msm.SCHEDULE_ID=ms.SCHEDULE_ID  WHERE mw.END_TIME <GETDATE() and ms.RECURRENCE_PATTERN = 0 and ms.RECURRENCE_PERIOD = 0 group by ms.schedule_id,ms.SCHEDULE_NAME,ms.DESCRIPTION";
  }
if ($sqltype eq 'ms')
  {
# 02/11/16    $sql = "select distinct cs.name, cs.ip, cs.origin,ms.schedule_id, ms.SCHEDULE_NAME, ms.DESCRIPTION, max(CONVERT(VARCHAR(50), ms.START_TIME,111)) as \'Start Date\', max(CONVERT(VARCHAR(50), ms.START_TIME,108)) as \'Start Time\', max(CONVERT(VARCHAR(50), mw.END_TIME, 111)) as \'End Date\', max(CONVERT(VARCHAR(50), mw.END_TIME, 108)) as \'End Time\' from CM_COMPUTER_SYSTEM cs INNER JOIN MAINTENANCE_SCHEDULE_MEMBERS msm  ON cs.cs_id=msm.CS_ID  INNER JOIN MAINTENANCE_WINDOW mw  ON msm.SCHEDULE_ID=mw.SCHEDULE_ID  INNER JOIN MAINTENANCE_SCHEDULE ms  ON msm.SCHEDULE_ID=ms.SCHEDULE_ID  WHERE mw.END_TIME <GETDATE() and ms.RECURRENCE_PATTERN = 0 and ms.RECURRENCE_PERIOD = 0 group by name,ip,origin,ms.schedule_id,ms.SCHEDULE_NAME,ms.DESCRIPTION";
    $sql = "select ms.schedule_id, ms.SCHEDULE_NAME, ms.DESCRIPTION, max(CONVERT(VARCHAR(50), ms.START_TIME,111)) as \'Start Date\', max(CONVERT(VARCHAR(50), ms.START_TIME,108)) as \'Start Time\', max(CONVERT(VARCHAR(50), mw.END_TIME, 111)) as \'End Date\', max(CONVERT(VARCHAR(50), mw.END_TIME, 108)) as \'End Time\' from CM_COMPUTER_SYSTEM cs with(nolock) INNER JOIN MAINTENANCE_SCHEDULE_MEMBERS msm with(nolock) ON cs.cs_id=msm.CS_ID  INNER JOIN MAINTENANCE_WINDOW mw with(nolock) ON msm.SCHEDULE_ID=mw.SCHEDULE_ID  INNER JOIN MAINTENANCE_SCHEDULE ms with(nolock) ON msm.SCHEDULE_ID=ms.SCHEDULE_ID  WHERE mw.END_TIME <GETDATE() and ms.RECURRENCE_PATTERN = 0 and ms.RECURRENCE_PERIOD = 0 group by ms.schedule_id,ms.SCHEDULE_NAME,ms.DESCRIPTION";
  }

# --- execute query
$sth = $dbh->prepare($sql);
$sth->execute or die "SQL Error: $DBI::errstr\n";

# --- loop in output
while (@row = $sth->fetchrow_array) 
 {
# - decode line (its normal that some variables are not used like they are common between several perls)
#  02/11/16 ($m_name,$m_ip,$m_origin,$m_calid,$m_schedule,$m_description,$m_startdate,$m_starttime,$m_enddate,$m_endtime) = @row;
  ($m_calid,$m_schedule,$m_description,$m_startdate,$m_starttime,$m_enddate,$m_endtime) = @row;
     $countm++;
     print "$countm - Cal_id: $m_calid Name: $m_schedule \n";
     print "    Start: $m_startdate $m_starttime End: $m_enddate $m_endtime\n";
 }

print "\n";
print "step 0: We found $countm expired calendars\n";

# --- disconnect db
$dbh->disconnect();

exit;
}


#
# step_dx - do we have parameter -dx"y"? (Delete expired calendars)
#

if ($delete_expired eq 'y')
{
# --- connect DB ---
&connect_db;

# --- prepare sql query (nimbus_type: 0= 1=robot 2=hub)
if ($sqltype eq 'my')
  {
    $sql = "";
    print "This function/option is not (yet) available for MySQL\n";
    exit 3;
  }
if ($sqltype eq 'ms')
  {
#    $sql = "DECLARE \@message2 varchar(100); DECLARE \@cs2 AS NVARCHAR(500); DECLARE plancursor CURSOR for select distinct ms.schedule_id from CM_COMPUTER_SYSTEM cs INNER JOIN MAINTENANCE_SCHEDULE_MEMBERS msm ON cs.cs_id = msm.CS_ID INNER JOIN MAINTENANCE_WINDOW mw ON msm.SCHEDULE_ID = mw.SCHEDULE_ID INNER JOIN MAINTENANCE_SCHEDULE ms ON msm.SCHEDULE_ID = ms.SCHEDULE_ID WHERE mw.END_TIME <GETDATE() and ms.RECURRENCE_PATTERN = 0 and ms.RECURRENCE_PERIOD = 0 group by name,ip,origin,ms.schedule_id,ms.SCHEDULE_NAME,ms.DESCRIPTION; open plancursor fetch next from plancursor into \@cs2 while \@\@FETCH_STATUS = 0 BEGIN print \' \' select \@message2 = \'-- Deleting member(s) from schedule_id: \' + \@cs2 + \' from MAINTENANCE_SCHEDULE_MEMBERS\'; print \@message2 delete from MAINTENANCE_SCHEDULE_MEMBERS where SCHEDULE_ID = \@cs2; select \@message2 = \'DELETING MAINTENANCE WINDOWS connected to schedule_id: \' + \@cs2 + \' from MAINTENANCE_WINDOW\' print \@message2 delete from MAINTENANCE_WINDOW where SCHEDULE_ID = \@cs# 2; select \@message2 = \'-- DELETING SCHEDULE ID: \' + \@cs2 + \' from MAINTENANCE_SCHEDULE\' print \@message2 delete from MAINTENANCE_SCHEDULE where SCHEDULE_ID = \@cs2; FETCH NEXT FROM plancursor into \@cs2 END CLOSE plancursor; DEALLOCATE plancursor;";

    $sql = "DECLARE \@message2 varchar(100)\; DECLARE \@cs2 AS NVARCHAR(500)\; DECLARE plancursor CURSOR for select distinct ms.schedule_id from CM_COMPUTER_SYSTEM cs with(nolock) INNER JOIN MAINTENANCE_SCHEDULE_MEMBERS msm with(nolock) ON cs.cs_id = msm.CS_ID INNER JOIN MAINTENANCE_WINDOW mw with(nolock) ON msm.SCHEDULE_ID = mw.SCHEDULE_ID INNER JOIN MAINTENANCE_SCHEDULE ms with(nolock) ON msm.SCHEDULE_ID = ms.SCHEDULE_ID WHERE mw.END_TIME <GETDATE() and ms.RECURRENCE_PATTERN = 0 and ms.RECURRENCE_PERIOD = 0\;  open plancursor fetch next from plancursor into \@cs2 while \@\@FETCH_STATUS = 0 BEGIN delete from MAINTENANCE_SCHEDULE_MEMBERS where SCHEDULE_ID = \@cs2\; delete from MAINTENANCE_WINDOW where SCHEDULE_ID = \@cs2\; delete from MAINTENANCE_SCHEDULE where SCHEDULE_ID = \@cs2\; FETCH NEXT FROM plancursor into \@cs2 END CLOSE plancursor\; DEALLOCATE plancursor\;";

  }

# --- execute query
$sth = $dbh->prepare($sql);
$sth->execute or die "SQL Error: $DBI::errstr\n";
$rows=$sth->rows;
if ($rows eq '1')
 {
   print "Deleted expired calendars from maintenance\n";
 } 

# --- disconnect db
$dbh->disconnect();

# print "Deleted expired calendars from maintenance\n";

exit;
}

#
# step1 - get all devices from CM_COMPUTER_SYSTEM and put this in table tab1 (so that we can give computername and get cs_id in return)
#


# --- connect DB ---
&connect_db;

# --- prepare sql query (nimbus_type: 0= 1=robot 2=hub)
if ($sqltype eq 'my')
  {
    $sql = "select name, cs_id, nimbus_type from CM_COMPUTER_SYSTEM where (nimbus_type = '1' or nimbus_type = '2' or nimbus_type = '0') order by name";
  }
else
  {
    $sql = "select name, cs_id, nimbus_type from CM_COMPUTER_SYSTEM with(nolock) where (nimbus_type = '1' or nimbus_type = '2' or nimbus_type = '0') order by name";
  }

# --- execute query
$sth = $dbh->prepare($sql);
$sth->execute or die "SQL Error: $DBI::errstr\n";

# --- loop in output
while (@row = $sth->fetchrow_array) 
 {
   
# - decode line
  ($db_name, $db_cs, $db_type) = @row;
   $count++;
# --- add found computer in table for later direct reference
#   print "Add server $count: $db_name csid: $db_cs in table\n";
# --- add robot in table
#   $tab1{$db_name}=$db_cs;
   $db_name=uc($db_name);
   $ty=$tab1{$db_name};
#   print "step 1 - We found cs_id: $db_cs for server: $db_name (saved cs_id: $ty)\n";
   if ($ty eq '')
    { 
      $tab1{$db_name}=$db_cs;
    }
   else
    { 
      $tab1{$db_name}="$ty\",\"$db_cs";
    } 
 }

print "step 1: We found $count devices in cm_computer_system table\n";

# --- disconnect db
$dbh->disconnect();

#
# step2a - read file with servers to be put in maintenance
#

if ($file_list eq '' && $input_group eq '' && $file_path ne '')
{
# --- open server input file
open (GLOBALS, "$file_path") or die "step 2a: Error: Can't open $file_path for read: $!";
@lines_server = <GLOBALS>;
close GLOBALS or die "step 2a: Error: Cannot close $file_path: $!"; 

# --- loop in servernames
foreach my $lineserver (@lines_server)
  {
    $reqcount++;
# --- remove blanks
    $lineserver =~ s/^\s+|\s+$//g;
    $lineserver=uc($lineserver);
    if ($display_server eq '')
     {
       $display_server=$lineserver;
     }
    else
     {
       if ($lineserver ne '')
        {
          $display_server="$display_server,$lineserver";
        }
     } 
# --- get cs_id stored in tab1 for server
    $o_csid=$tab1{$lineserver};
    if ($o_csid ne '')
     {  
         $cscount++;
         print "step 2a: server from table: $lineserver has cs_id: $o_csid\n"; 
        if ($cslist eq '')
         {
           $cslist="\"$o_csid\"";
         }
        else
         {
           $cslist="$cslist,\"$o_csid\"";
         }
#       print "cslist: $cslist\n"; 
     }
    else
     {
       print "step 2a: Error: not found $lineserver (in CM_COMPUTER_TABLE)\n";
     }
  }

print "step 2a: We added $cscount servers to $action_text in maintenance (requested: $reqcount servers)\n";
}

#
# step2b - read -h with server(s) to be put in maintenance
#

if ($file_list ne '')
{
# --- split input parameter $file_list
  @rep = split(/,/ , $file_list);
  $repcount=@rep;
  for (my $i=0; $i <= $repcount; $i++) 
   {
    $lineserver=$rep[$i];
    $lineserver=uc($lineserver);
    if ($display_server eq '')
     {
       $display_server=$lineserver;
     }
    else
     {
       if ($lineserver ne '')
        {
           $display_server="$display_server,$lineserver";
        }
     } 
# --- remove blanks
      $lineserver =~ s/^\s+|\s+$//g; 
      $lineserver=uc($lineserver);
      if ($lineserver ne '')
      {
      $reqcount++;
# --- get cs_id stored in tab1 for server
      $o_csid=$tab1{$lineserver};
      if ($o_csid ne '')
       {  
        $cscount++;
        print "step 2b: server $reqcount from table: $lineserver has cs_id(s): \"$o_csid\"\n"; 
        if ($cslist eq '')
         {
           $cslist="\"$o_csid\"";
         }
        else
         {
           $cslist="$cslist,\"$o_csid\"";
         }
#       print "cslist: $cslist\n"; 
       }
      else
       {
        print "step 2b: Error: not found $lineserver (in CM_COMPUTER_TABLE)\n";
       }
      }
   }

print "step 2b: We added $cscount servers to $action_text in maintenance\n";

# -- if no servers found stop processing
if ($cscount eq '0')
 {
   print "step 2b: Stop processing because no servers are selected\n";
   exit 1;
 }
}

#
# step2c - select servers from an USM group
#

if ($input_group ne '')
{
  $sinc=0;
  $sexc=0;
# --- connect DB ---
  &connect_db;

# --- prepare sql query (nimbus_type: 0= 1=robot 2=hub)
if ($sqltype eq 'my')
  {
#  $sql = "select s.ip, s.os_type,s.os_name, r.domain,r.hub,r.robot,r.origin,is_hub,r.robot_active,r.user_tag_1,r.user_tag_2,r.address,g.name \'group\', s.name \'group_member\' from CM_COMPUTER_SYSTEM s, CM_GROUP g, CM_GROUP_MEMBER m,CM_NIMBUS_ROBOT r where g.grp_id = m.grp_id and m.cs_id = s.cs_id and s.name = robot and os_type is not NULL and g.name = \'$input_group\'";
      $sql="Select s.ip, s.os_type,s.os_name,r.domain,r.hub,r.robot,r.origin,is_hub,r.robot_active,r.user_tag_1,r.user_tag_2,r.address,g.name \'group\',s.name \'group_member\' from CM_GROUP g WITH (NOLOCK),CM_GROUP_MEMBER m WITH (NOLOCK),CM_COMPUTER_SYSTEM s WITH (NOLOCK) left outer join cm_nimbus_robot r on r.robot = s.name where g.grp_id=(select grp_id from CM_GROUP WITH (NOLOCK) where name=\'$input_group\') AND g.grp_id = m.grp_id and m.cs_id = s.cs_id ";
  }
else
  {
      $sql="Select s.ip, s.os_type,s.os_name,r.domain,r.hub,r.robot,r.origin,is_hub,r.robot_active,r.user_tag_1,r.user_tag_2,r.address,g.name \'group\',s.name \'group_member\' from CM_GROUP g WITH (NOLOCK),CM_GROUP_MEMBER m WITH (NOLOCK),CM_COMPUTER_SYSTEM s WITH (NOLOCK) left outer join cm_nimbus_robot r with(nolock) on r.robot = s.name where g.grp_id=(select grp_id from CM_GROUP WITH (NOLOCK) where name=\'$input_group\') AND g.grp_id = m.grp_id and m.cs_id = s.cs_id ";
  }
# --- execute query
  $sth = $dbh->prepare($sql);
  $sth->execute or die "SQL Error: $DBI::errstr\n";

# --- loop in output
  while (@row = $sth->fetchrow_array) 
   {
# - decode line
     ($o_ip,$o_os_major,$o_os_minor,$o_domain,$o_hub,$o_robot,$o_origin,$o_is_hub,$o_robot_active,$o_user_tag1,$o_user_tag2,$o_address,$o_group,$lineserver) = @row;
     $count++;
# - do we have s_dev_inc/exc
     if ($s_dev_inc ne '')
      {
        if ($lineserver =~ /$s_dev_inc/i ) 
           {
             $domatch=1;
           }
         else
           {
             if ($debug eq '2') { print "Excluded robot: $lineserver due to device include (for: $o_address)\n"; }
             $count_robot_exclude++;
             $sexc++;
             goto bypassg;
           }
      }  
     if ($s_dev_exc ne '')
      {
        if ($lineserver =~ /$s_dev_exc/i ) 
           {
             if ($debug eq '2') { print "Excluded robot: $lineserver due to device exclude (for: $o_address)\n"; }
             $count_robot_exclude++;
             $sexc++;
             goto bypassg;
           }
      } 
# --- do we bypass hub robots?
       if ($bypass_hub eq 'y')
        {
          if ($o_is_hub ne '1' ) 
             {
               $domatch=1;
             }
           else
             {
               if ($debug eq '2') { print "Excluded bypass_hub: $o_is_hub due to hub robot (for: $o_address)\n"; }
               $count_ishub_exclude++;
               $sexc++;
               goto bypassg;
             }
        }  
     $sinc++;
     print "Step 2c: Included device $sinc: $lineserver\n";
     if ($display_server eq '')
      {
        $display_server=$lineserver;
      }
     else
      {
        if ($lineserver ne '')
         {
            $display_server="$display_server,$lineserver";
         }
      } 
# --- remove blanks
      $lineserver =~ s/^\s+|\s+$//g; 
      $lineserver=uc($lineserver);
      if ($lineserver ne '')
      {
      $reqcount++;
# --- get cs_id stored in tab1 for server
      $o_csid=$tab1{$lineserver};
      if ($o_csid ne '')
       {  
        $cscount++;
#        print "step 2c: server $reqcount from table: $lineserver has cs_id: $o_csid \n"; 
        if ($cslist eq '')
         {
           $cslist="\"$o_csid\"";
         }
        else
         {
           $cslist="$cslist,\"$o_csid\"";
         }
#       print "cslist: $cslist\n"; 
       }
      else
       {
        print "step 2c: Error: not found $lineserver from USM group: $input_group (in CM_COMPUTER_TABLE)\n";
       }
      }
bypassg:
  }

print "step 2c: We found $cscount servers to $action_text from USM group: $input_group (and excluded: $sexc)\n";

# --- disconnect db
$dbh->disconnect();

# -- if no servers found stop processing
if ($cscount eq '0')
 {
   print "step 2c: Stop processing because no servers are selected\n";
   exit 1;
 }
}

#
# step2d - select servers from an cm_nimbus_robot
#

if ($serverregex eq 'y' && $input_group eq '')
{
# --- connect DB ---
  &connect_db;

# --- prepare sql query (nimbus_type: 0= 1=robot 2=hub)
if ($sqltype eq 'my')
  {
    if ($file_list_regex ne '')
      {
         $sql = "select name, cs_id, nimbus_type from CM_COMPUTER_SYSTEM where (nimbus_type = '1' or nimbus_type = '2' or nimbus_type = '0') order by name";
      }
    else
      {
        $sql = "SELECT ip,os_major,os_minor,domain,hub,robot,origin,is_hub,robot_active,user_tag_1,user_tag_2,address,dev_name FROM CM_NIMBUS_ROBOT r,CM_DEVICE d where r.dev_id = d.dev_id";
      } 
  }
else
  {
    if ($file_list_regex ne '')
      { 
         $sql = "select name,  cs_id, nimbus_type from CM_COMPUTER_SYSTEM with(nolock) where (nimbus_type = '1' or nimbus_type = '2' or nimbus_type = '0') order by name";
      }
    else
      { 
         $sql = "SELECT ip,os_major,os_minor,domain,hub,robot,origin,is_hub,robot_active,user_tag_1,user_tag_2,address,dev_name FROM CM_NIMBUS_ROBOT r with(nolock),CM_DEVICE d with(nolock) where r.dev_id = d.dev_id";
      }
  }

# --- execute query
  $sth = $dbh->prepare($sql);
  $sth->execute or die "SQL Error: $DBI::errstr\n";

# --- loop in output
  while (@row = $sth->fetchrow_array) 
   {
# - decode line
    if ($file_list_regex eq '') {  ($o_ip,$o_os_major,$o_os_minor,$o_domain,$o_hub,$o_robot,$o_origin,$o_is_hub,$o_robot_active,$o_user_tag1,$o_user_tag2,$o_address,$o_dev_name) = @row;}
    if ($file_list_regex ne '') {  ($o_dev_name,$o_cs_id,$o_type) = @row;}
     $count++;

# --- do we have hub_inc or hub_exc
  if ($s_hub_inc ne '')
    {
      if ($o_hub =~ /$s_hub_inc/i ) 
         {
           $domatch=1;
         }
      else
         {
           if ($debug eq '2') { print "Excluded hub: $o_hub due to hub include (for: $o_address)\n"; }
           $count_hub_exclude++;
           goto bypass;
         }
    } 
  if ($s_hub_exc ne '')
    {
      if ($o_hub =~ /$s_hub_exc/i ) 
         {
           if ($debug eq '2') { print "Excluded hub: $o_hub due to hub exclude (for: $o_address)\n"; }
           $count_hub_exclude++;
           goto bypass;
         }
    } 

# --- do we have s_hub_origin include
     if ($s_hub_origin_inc ne '')
      {
        if ($o_origin =~ /$s_hub_origin_inc/i ) 
           {
             $domatch=1;
           }
         else
           {
             if ($debug eq '2') { print "Excluded origin: $o_origin due to origin include (for: $o_address)\n"; }
             $count_origin_exclude++;
             goto bypass;
           }
      }  
     if ($s_hub_origin_exc ne '')
      {
        if ($o_origin =~ /$s_hub_origin_exc/i ) 
           {
             if ($debug eq '2') { print "Excluded origin: $o_origin due to origin exclude (for: $o_address)\n"; }
             $count_origin_exclude++;
             goto bypass;
           }
      }   

# --- do we have -ir device regex ($file_list_regex)
     if ($file_list_regex ne '')
       {
        if ($o_dev_name =~ /$file_list_regex/i ) 
           {
             $domatch=1;
           }
         else
           {
             if ($debug eq '2') { print "Excluded device: $o_dev_name due to device regex (for: $o_address)\n"; }
             $count_dev_exclude++;
             goto bypass;
           }
      }  

# --- do we have s_dev_inc include
     if ($s_dev_inc ne '')
      {
#        print "regex: $s_dev_inc name: $o_dev_name\n";
        if ($o_dev_name =~ /$s_dev_inc/i ) 
           {
             $domatch=1;
           }
         else
           {
             if ($debug eq '2') { print "Excluded device: $o_dev_name due to device include (for: $o_address)\n"; }
             $count_dev_exclude++;
             goto bypass;
           }
      }  
     if ($s_dev_exc ne '')
      {
        if ($o_dev_name =~ /$s_dev_exc/i ) 
           {
             if ($debug eq '2') { print "Excluded device: $o_dev_name due to device exclude (for: $o_address)\n"; }
             $count_dev_exclude++;
             goto bypass;
           }
      } 

# --- do we have s_robot_inc include
     if ($s_robot_inc ne '')
      {
        if ($o_robot =~ /$s_robot_inc/i ) 
           {
             $domatch=1;
           }
         else
           {
             if ($debug eq '2') { print "Excluded robot: $o_robot due to robot include (for: $o_address)\n"; }
             $count_robot_exclude++;
             goto bypass;
           }
      }  
     if ($s_robot_exc ne '')
      {
        if ($o_robot =~ /$s_robot_exc/i ) 
           {
             if ($debug eq '2') { print "Excluded robot: $o_robot due to robot exclude (for: $o_address)\n"; }
             $count_robot_exclude++;
             goto bypass;
           }
      } 

# --- do we have s_robot_user1 include
     if ($s_robot_user1 ne '')
      {
        if ($o_user_tag1 =~ /$s_robot_user1/i ) 
           {
             $domatch=1;
           }
         else
           {
             if ($debug eq '2') { print "Excluded user_tag_1: $o_user_tag_1 due to user1 include (for: $o_address)\n"; }
             $count_user_tag1_exclude++;
             goto bypass;
           }
      }

# --- do we have s_robot_user2 include
     if ($s_robot_user2 ne '')
      {
        if ($o_user_tag2 =~ /$s_robot_user2/i ) 
           {
             $domatch=1;
           }
         else
           {
             if ($debug eq '2') { print "Excluded user_tag_2: $o_user_tag_2 due to user2 include (for: $o_address)\n"; }
             $count_user_tag2_exclude++;
             goto bypass;
           }
      }  

# --- do we bypass hub robots?
       if ($bypass_hub eq 'y')
        {
          if ($o_is_hub ne '1' ) 
             {
               $domatch=1;
             }
           else
             {
               if ($debug eq '2') { print "Excluded bypass_hub: $o_is_hub due to hub robot (for: $o_address)\n"; }
               $count_ishub_exclude++;
               goto bypass;
             }
        }   
 
     if ($domatch eq '1')
       {
         $selected++;
         $lineserver=$o_dev_name;  
         print "- $selected: Included $lineserver (hub: $o_hub origin: $o_origin)\n";
      }

       if ($display_server eq '')
        {
          $display_server=$lineserver;
        }
       else
        {
          if ($lineserver ne '')
           {
              $display_server="$display_server,$lineserver";
           }
        } 
# --- remove blanks
      $lineserver =~ s/^\s+|\s+$//g; 
      $lineserver=uc($lineserver);
      if ($lineserver ne '')
      {
      $reqcount++;
# --- get cs_id stored in tab1 for server
      $o_csid=$tab1{$lineserver};
      if ($o_csid ne '')
       {  
        $cscount++;
#        print "step 2d: server $reqcount from table: $lineserver has cs_id: $o_csid \n"; 
        if ($cslist eq '')
         {
           $cslist="\"$o_csid\"";
         }
        else
         {
           $cslist="$cslist,\"$o_csid\"";
         }
#       print "cslist: $cslist\n"; 
       }
      else
       {
        print "step 2d: Error: not found $lineserver from USM group: $input_group (in CM_COMPUTER_TABLE)\n";
       }
      }
bypass:
  }

print "step 2d: We added $cscount servers to $action_text from regex selections\n";

# --- disconnect db
$dbh->disconnect();

# -- if no servers found stop processing
if ($cscount eq '0')
 {
   print "step 2d: Stop processing because no servers are selected\n";
   exit 1;
 }
}

if ($simulate eq 'y')
{
  print "step 2d: *** Stop execution due to simulation run (-sim\"y\") *** \n";
  exit;
}

#
# step3: define the calendar
#

if ($action eq 'c')
 {
   &define_cal;
# - as result we have the calendar id in: $schid
  $scheduleid=$schid;

  print "step 3: We defined a calendar ($cal_title) with id: $schid\n";
 } 

#
# step4: execute restful command
#

# - Set servername
$ServerAndPort = $Server_With_Rest_Interface . ":" . $RestServicePort;

# - Define the query URL for UIM RESTful interface
if ($action eq 'a' || $action eq 'c')
 {
#   $baseURL = "$use_https://" . $Server_With_Rest_Interface . "/rest/maintenance_mode/$UIM_Domain/$UIM_Hub/$UIM_Robot/add_computer_systems_to_schedule/$scheduleid";
   $baseURL = "$use_https://" . $ServerAndPort . "/rest/maintenance_mode/$UIM_Domain/$UIM_Hub/$UIM_Robot/add_computer_systems_to_schedule/$scheduleid";
 }
else
 {
#   $baseURL = "$use_https://" . $Server_With_Rest_Interface . "/rest/maintenance_mode/$UIM_Domain/$UIM_Hub/$UIM_Robot/remove_computer_systems_from_schedule/$scheduleid";
   $baseURL = "$use_https://" . $ServerAndPort . "/rest/maintenance_mode/$UIM_Domain/$UIM_Hub/$UIM_Robot/remove_computer_systems_from_schedule/$scheduleid";
 }
if ($debug eq 'y') { print "..  Post action: $action action_text: $action_text url: $baseURL\n";}
 
# - Create LWP Object
$RestConnection = LWP::UserAgent->new();

# - Create connection
$realm = "NimJAAS";

if ($debug eq 'y') { print "..  serverport: $ServerAndPort realm: $realm user: $username pass: $password\n"; }

$RestConnection->credentials($ServerAndPort, $realm, $username, $password);
# $response=$RestConnection->credentials($ServerAndPort, $realm, $username=>$password);
# print "step4: initial rest connection: $response\n";

# - to avoid https certificate errors
$RestConnection->ssl_opts( verify_hostname => 0 ,SSL_verify_mode => 0x00);

# - Create POST data
$post_data = "{ \"cs\":\[$cslist\] }";
if ($debug eq 'y') {  print "..  We will POST: $post_data \n"; }

# print "step 4a: We will post the $action_text request to server: $Server_With_Rest_Interface\n";
print "step 4a: We will post the $action_text request to server: $ServerAndPort\n";

$req = HTTP::Request->new(POST => $baseURL);
$req->content_type('application/json');
$req->content($post_data);

# - Execute POST        
$resp = $RestConnection->request($req);

# - Decode response
if ($resp->is_success) {
    $message = $resp->decoded_content;
    if ($message eq '')
     {
       $message="ok";
       $rc=0;
     } 
    print "step 4b: Received $action_text reply: $message\n";
}
else 
{
    $rc=$resp->code;
    print "step 4b: HTTP POST error code: ", $resp->code, "\n";
    print "step 4b: HTTP POST error message: ", $resp->message, "\n";
}

exit $rc;

#
# --- sub procedures ---
#

sub connect_db
{
# --- connect DB
# $dbh = DBI->connect("DBI:ODBC:Driver={SQL Server};Server=$sql_server;Database=$sql_db;UID=$sql_user;PWD=$sql_password") or die("\n\nCONNECT ERROR:\n\n$DBI::errstr");

# --- connect DB ---
if ($sqltype eq 'my')
  {
    $dbh = DBI->connect("DBI:mysql:database=$sql_db;host=$sql_server", "$sql_user", "$sql_password") || die "Could not connect to $database database on $sqlserver: $DBI::errstr";
  }
if ($sqltype eq 'ms')
  {
    if ($trusted eq 'yes')
      {
        $dbh = DBI->connect("DBI:ODBC:Driver={SQL Server};Server=$sql_server;Database=$sql_db;Trusted_Connection=Yes;") or die("\n\nCONNECT ERROR:\n\n$DBI::errstr");
#        print "Using mssql trusted connection to connect to: $sql_server\n";
      }
    else
      {
        $dbh = DBI->connect("DBI:ODBC:Driver={SQL Server};Server=$sql_server;Database=$sql_db;UID=$sql_user;PWD=$sql_password") or die("\n\nCONNECT ERROR:\n\n$DBI::errstr");
#        print "Connecting to MSSQL server: $sql_server db: $sql_db user: $sql_user\n";
      }
  }
}

sub prep_var
{
# --- get credentials and settings from common file
($Server_With_Rest_Interface, $RestServicePort, $username, $password,$UIM_Domain,$UIM_Hub,$UIM_Robot,$sql_server,$sql_user,$sql_password,$sql_db,$sql_type,$use_https) = nimsoft_generic::getRestData();
#print "dat: sql_user: $sql_user sql_type: $sql_type sql_db: $sql_db\n";
if ($sql_type eq '') { $sql_type="mssql";}
# -- de-crypt (need $encoded)
$decoded = decode_base64($password);
$password = RC4($crkey, $decoded);

$decoded = decode_base64($sql_password);
$sql_password = RC4($crkey, $decoded);


if ($sql_type eq 'mssql')
 {
    $sqltype="ms";
    if ($sql_user eq 'trusted')
     {
        $trusted="yes";
        $sql_password=""; 
#        print "We will use a trusted connection\n";
     }
	else
     {
#       print "We will connect to MSSQL via user and password\n";
     }	   
 } 
if ($sql_type eq 'mysql')
 {
    $sqltype="my";
 } 

# --- variables
# - table for our found computers in cm_computer_table
@tab1 = ();

# - counter for read computers
$count=0;

# - variable to create cs_id list
$cslist="";
# - counter for number of cs_ids added
$cscount=0;
# - counter for the number of servers requested to put in maintenance
$reqcount=0;
# - counter for schedules
$counts=0;
# - counter for hosts in maintenance
$countm=0;
@mdays = (0,31,28,31,30,31,30,31,31,30,31,30,31);

if ($debug eq 'y') 
 {
   print "..  From dir: $curdir dat: nimsoft_generic.dat\n";
   print "..  server: $Server_With_Rest_Interface, http: $use_https, port: $RestServicePort, user: $username, password: $password, domain: $UIM_Domain, hub: $UIM_Hub, robot: $UIM_Robot, sqlserver: $sql_server, sqluser: $sql_user, sqlpass: $sql_password, sqldb: $sql_db\n";
 }
}

sub calcdate
 {
# --- get time + 1 minutes 
 if ($duration_from eq '')
  {  
    ($d_sec,$d_min,$d_hour,$d_day,$d_month,$d_year,$wday,$yday,$dntcare)=localtime(time + 60); 
     $d_year=$d_year+1900;
     $d_month=$d_month+1; 
     $display_from=sprintf("%02d-%02d-%02d %02d:%02d:%02d",$d_year,$d_month,$d_day,$d_hour,$d_min,$d_sec);
     print "Today its: $display_from \n";
     print "\n";
  }
 }

sub validate 
{
# if ($ck =~ m!^((?:19|20)\d\d)[- /.](0[1-9]|1[012])[- /.](0[1-9]|[12][0-9]|3[01])$!) 
 if ($ck =~ m!^((?:19|20)\d\d)[- /.](0[1-9]|1[012])[- /.](0[1-9]|[12][0-9]|3[01]).([01]?[0-9]|2[0-3]):([0-5][0-9]):([0-5][0-9])$!) 
  {
# - At this point, $1 holds the year, $2 the month and $3 the day of the date entered
    if ($3 == 31 and ($2 == 4 or $2 == 6 or $2 == 9 or $2 == 11)) 
      {
        return 0; # 31st of a month with 30 days
      } 
    elsif ($3 >= 30 and $2 == 2) 
      {
        return 0; # February 30th or 31st
      } 
    elsif ($2 == 2 and $3 == 29 and not ($1 % 4 == 0 and ($1 % 100 != 0 or $1 % 400 == 0))) 
      {
       return 0; # February 29th outside a leap year
      } 
    else 
     {
       return 1; # Valid date
     }
  } 
 else 
  {
    return 0; # Not a date
  }
}

sub define_cal
{
if ($cal_title eq '' && $input_group eq '')
 {
   $cal_title=$display_server;
 }
if ($cal_title eq '' && $input_group ne '')
 {
   $cal_title="Group_$input_group";
 }
# --- Overview
  print "\n";
  print "We will put in maintenance:\n";
  print "   calendar title: $cal_title\n";
  print "   servers: $display_server\n"; 
  if ($duration_from eq '')
   {
    print "   from: now + 1 minutes\n";
   }
  else
   { 
    print "   from: $duration_until\n";
    print "   (yyyy: $d_year mm: $d_month dd: $durf_day hour: $d_hour min: $d_min sec: $d_sec)\n";
   }
  if ($duration_seconds ne '')
   {
     print "   duration: $duration_seconds\n"; 
     print "   (hours: $s_hours minutes: $s_minutes seconds: $s_seconds)\n";
     print "   from: $display_from\n"; 
   }
  if ($duration_until ne '')
   {
    print "   duration: $duration_until\n";
    print "   (yyyy: $durd_yyyy mm: $durd_mm dd: $durd_dd hour: $durt_hour min: $durt_min sec: $durt_sec)\n";
   }
  print "\n";

# --- end overview

# - Set servername
my $ServerAndPort = $Server_With_Rest_Interface . ":" . $RestServicePort;

# Define the query URL for UIM RESTful interface
# my $baseURL = "$use_https://" . $Server_With_Rest_Interface . "/rest/maintenance_mode/$UIM_Domain/$UIM_Hub/$UIM_Robot/add_schedule";
my $baseURL = "$use_https://" . $ServerAndPort . "/rest/maintenance_mode/$UIM_Domain/$UIM_Hub/$UIM_Robot/add_schedule";

if ($debug eq 'y') { print "url: $baseURL\n";}

# - Create LWP Object
my $RestConnection = LWP::UserAgent->new();

# - Create connection
my $realm = "NimJAAS";
$RestConnection->credentials($ServerAndPort, $realm, $username, $password);

# - to avoid https certificate errors
$RestConnection->ssl_opts( verify_hostname => 0 ,SSL_verify_mode => 0x00);

my $req = HTTP::Request->new(POST => $baseURL);
$req->content_type('application/json');
if ($duration_until eq '')
 {
   $desc1="Duration: $duration_seconds seconds From: $display_from";
 }
else
  {
   $desc1="From: $display_from Until: $duration_until";
 }



# - if seconds: type=duration if end_time: type=end_date_time if date, type=datetime
if ($duration_until eq '')
{
$post_data = "{
  \"name\": \"$cal_title\",
  \"description\": \"$desc1\",
  \"start_date_time\": {
    \"month\": \"$d_month\",
    \"day\": \"$d_day\",
    \"year\": \"$d_year\",
    \"timestamp\": {
      \"hours\": \"$d_hour\",
      \"minutes\": \"$d_min\",
      \"seconds\": \"$d_sec\"
    }
  },
  \"end_time\": {
    \"type\": \"duration\",
    \"end_date_time\": {
      \"month\": \"$durd_mm\",
      \"day\": \"$durd_dd\",
      \"year\": \"$durd_yyyy\",
      \"timestamp\": {
        \"hours\": \"$durt_hour\",
        \"minutes\": \"$durt_min\",
        \"seconds\": \"$durt_sec\"
      }
    },
    \"duration\": {
      \"hours\": \"$s_hours\",
      \"minutes\": \"$s_minutes\",
      \"seconds\": \"$s_seconds\"
    }
  },
  \"timezone\": \"Europe/Brussels\"
 }";
}
else
{
$post_data = "{
  \"name\": \"$cal_title\",
  \"description\": \"$desc1\",
  \"start_date_time\": {
    \"month\": \"$d_month\",
    \"day\": \"$d_day\",
    \"year\": \"$d_year\",
    \"timestamp\": {
      \"hours\": \"$d_hour\",
      \"minutes\": \"$d_min\",
      \"seconds\": \"$d_sec\"
    }
  },
  \"end_time\": {
    \"type\": \"end_date_time\",
    \"end_date_time\": {
      \"month\": \"$durd_mm\",
      \"day\": \"$durd_dd\",
      \"year\": \"$durd_yyyy\",
      \"timestamp\": {
        \"hours\": \"$durt_hour\",
        \"minutes\": \"$durt_min\",
        \"seconds\": \"$durt_sec\"
      }
    },
    \"duration\": {
      \"hours\": \"$s_hours\",
      \"minutes\": \"$s_minutes\",
      \"seconds\": \"$s_seconds\"
    }
  },
  \"timezone\": \"Europe/Brussels\"
 }";
}

if ($debug eq 'y') { print "$post_data\n"; }

$req->content($post_data);

my $sendResp = $RestConnection->request($req);

# print Dumper($sendResp);

if ($sendResp->is_success) 
 {
   my $parser = XML::LibXML->new(); 
   my $alarmlist = $parser->load_xml(string=>$sendResp->content);
# -- or we do a find on a level with sublevels and use a foreach, OR we can do a search on e detail level and get the value directly
  foreach my $alarm ($alarmlist->findnodes('/schedule'))
    {
#      print "alarm: $alarm\n";
      $schid=$alarm ->findvalue('./schedule_id');
#      print "Saved schid libxml: $schid\n";
    }
 }
else 
 {
     print STDERR "ERROR:", $sendResp->status_line, "\n";
 }
}

# --- simple check used in startup to know if REST services are already up and running, else we will wait
sub check_rest
{
$hubcount=0;

# - Set servername
$ServerAndPort = $Server_With_Rest_Interface . ":" . $RestServicePort;

# Define the query URL for UIM RESTful interface
# $baseURL = "$use_https://" . $Server_With_Rest_Interface . "/rest/hubs";
$baseURL = "$use_https://" . $ServerAndPort . "/rest/hubs";

if ($debug eq 'y') { print "url: $baseURL\n";}

# - Create LWP Object
$RestConnection = LWP::UserAgent->new();

# - Create connection
$realm = "NimJAAS";
$RestConnection->credentials($ServerAndPort, $realm, $username, $password);

# - to avoid https certificate errors
$RestConnection->ssl_opts( verify_hostname => 0 ,SSL_verify_mode => 0x00);

$searchResp = $RestConnection->get($baseURL);

# print Dumper($searchResp);

if ($searchResp->is_success) 
 {
  my $parser = XML::LibXML->new(); 
  my $hublist = $parser->load_xml(string=>$searchResp->content);
  foreach my $alarm ($hublist->findnodes('/hublist/hub'))
   {
    $hubcount++;
    $a_hub=$alarm ->findvalue('./name');
    $a_robot=$alarm ->findvalue('./robotName');
    if ($debug eq 'y') { print "$hubcount hub: $a_hub robot: $a_robot\n";}
   }
   print "Rest_Check - REST services are responding\n";
   print "\n";
 }
else 
 {
   print "Rest_Check - ERROR:", $searchResp->status_line, "\n";
   $errmsg=$searchResp->message;
   if ($errmsg eq 'Not Found')
    {
      $errmsg="Wasp Rest Services are not responding (anymore)";
   }
  print "Rest_Check - Wasp Resp services are not ready (yet) msg: $errmsg\n";
  exit 404;
 }
}

sub getparams
{
  usage() unless @ARGV gt 0;
  $argCount=$#ARGV + 1;
  for ($i=0 ; $i < $argCount ; $i++) 
  {
	if ($ARGV[$i] =~ /^-\?/)
	{ 
		usage();
	} 
   
	if ($ARGV[$i] =~ /^-lc/ ) 
	{ 
		$list_calendar =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-lm/ ) 
	{ 
		$list_maint =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-ls/ ) 
	{ 
		$ismachine =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-le/ ) 
	{ 
		$list_empty =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-de/ ) 
	{ 
		$delete_empty =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-lx/ ) 
	{ 
		$list_expired =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-dx/ ) 
	{ 
		$delete_expired =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-dm/ ) 
	{ 
		$delete_maint =  substr($ARGV[$i],3);
        }        
	if ($ARGV[$i] =~ /^-ci/ ) 
	{ 
		$scheduleid =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-if/ ) 
	{ 
		$file_path =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-ih/ ) 
	{ 
		$file_list =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-ir/ ) 
	{ 
		$file_list_regex =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-ig/ ) 
	{ 
		$input_group =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-db/ ) 
	{ 
		$debug =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-ac/ ) 
	{ 
		$action =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-cs/ ) 
	{ 
		$duration_seconds =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-cf/ ) 
	{ 
		$duration_from =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-cu/ ) 
	{ 
		$duration_until =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-ct/ ) 
	{ 
		$cal_title =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-ti/ )   
	{ 
		$s_robot_inc =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-te/ )   
	{ 
		$s_robot_exc =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-hi/ )   
	{ 
		$s_hub_inc =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-he/ )   
	{ 
		$s_hub_exc =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-ni/ )   
	{ 
		$s_dev_inc =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-ne/ )   
	{ 
		$s_dev_exc =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-ho/ )   
	{ 
		$s_hub_origin_inc =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-hx/ )   
	{ 
		$s_hub_origin_exc =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-ro/ )   
	{ 
		$s_robot_origin_inc =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-rx/ )   
	{ 
		$s_robot_origin_exc =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-u1/ )   
	{ 
		$s_robot_user1 =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-u2/ )   
	{ 
		$s_robot_user2 =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-bh/ )   
	{ 
		$bypass_hub =  substr($ARGV[$i],3);
        }
	if ($ARGV[$i] =~ /^-sim/ )   
	{ 
		$simulate =  substr($ARGV[$i],4);
        }
 }

  if ($list_calendar eq '')
   {  
     $list_calendar="n";
   }
  if ($list_empty eq '')
   {  
     $list_empty="n";
   } 
  if ($list_expired eq '')
   {  
     $list_expired="n";
   } 
  if ($delete_expired eq '')
   {  
     $delete_expired="n";
   }  
  if ($delete_maint eq '')
   {  
     $delete_maint="n";
   }  
  if ($delete_empty eq '')
   {  
     $delete_empty="n";
   }    
  if ($debug eq '')
   {  
     $debug="n";
   } 
  if ($list_maint eq '')
   {  
     $list_maint="n";
   } 
  if ($action eq '' && $ismachine eq '')
  {
    $action="c";
  }
#  if ($file_path eq '' && $input_group eq '' && $list_calendar eq 'n' && $list_maint eq 'n' && $file_list eq '' && $ismachine eq '' && $list_empty ne 'y' && $list_expired ne 'y' && $delete_expired ne 'y')
#   {
#      print "You MUST define the inputfile with a list of servers to put in maintenance with option -if (Input File) or -ih (Input Host)\n";
#      print "1: $file_path 2: $list_calendar 3: $list_maint 4: $file_list 5: $ismachine 6: $list_empty 7: $list_expired 8: $delete_expired\n";
#      exit 2;
#   }
  if ($bypass_hub eq '')
   {
     $bypass_hub="y";
   }
  if ($list_calendar eq 'n' && $list_maint eq 'n' && $ismachine eq '' && $list_empty ne 'y' && $list_expired ne 'y' && $delete_expired ne 'y' && $delete_empty ne 'y')
   {
     $serverlist="n";
   }
  else
   {
     $serverlist="y";
   } 
  if ($scheduleid eq '' && $action eq 'a')
   {
      print "You MUST define the calendar id with option -ci (use -lc to obtain the calendar ids)\n";
      exit 3;
   }
  if ($action ne 'a' && $action ne 'r' && $action ne 'c' && $ismachine eq '')
   {
      print "You need to define the action with parameter -ac\n";
      exit 0;
   }

# - simulate=y by default
  if ($simulate eq '') 
   { 
     $simulate="y";
   }

# - do we have server regex selection?
  if ($file_list_regex ne '' || $s_hub_inc ne '' || $s_hub_exc ne '' || $s_dev_inc ne '' || $s_dev_exc ne '' || $s_hub_origin_inc ne '' || $s_hub_origin_exc ne '' || $s_robot_inc ne '' || $s_robot_exc ne '' || $s_robot_user1 ne '' || $s_robot_user2 ne '')
   {
     $serverregex="y";
     if ($cal_title eq '')
      {
        print "When selecting the servers via regex selection you must give a calendar title via -ct\n";
        exit;
      }
    if (($input_group ne '' || $file_path ne '' || $file_list ne '') && ($s_dev_inc eq '' && $s_dev_exc eq ''))
      {
        print "When selecting the servers via regex selection you cannot use: -if (file), -ih (servers) or -ig (group)\n";
        exit;
      }     
   }
  else
   {
     if ($simulate eq '') 
      { 
        $simulate="n";
      }
# - do we have server selection?
    if (($input_group eq '' && $file_path eq '' && $file_list eq '' && $file_list_regex eq '' && $delete_maint eq 'n' ) && $serverlist ne 'y')
      {
        print "You must select servers via regex selections or via: -if (file), -ih (servers) or -ig (group)\n";
        exit;
      }
    else
      {
          if ($input_group ne '' || $file_path ne '' || $file_list ne '' || $file_list_regex ne '')
            {
             $serverselect="y";
            }
      }
   }

# - if servers are selected, do we have calendar definitions
   if (($serverselect eq 'y' || $serverregex eq 'y') && $action eq 'c')
    {
      if ($duration_seconds eq '' && $duration_from eq '' && $duration_until eq '')
        {
          print "If you select servers to be added in a new calendar you need to define calendar duration criteria (-cf, -cu or -cs)\n";
          exit;
        }
    }
   if (($serverselect eq 'y' || $serverregex eq 'y') && $action eq 'a')
    {
      if ($scheduleid eq '')
        {
          print "If you want servers added into an existing calendar you must define the calendar_id: -ci\n";
          exit;
        }
    }


  if ($action eq 'a') {$action_text="add server";} 
  if ($action eq 'r') {$action_text="remove server(s)";} 
  if ($action eq 'c') {$action_text="define server(s)";} 

if ($action eq 'r')
 {
      if ($scheduleid eq '')
        {
          print "If you want servers removed from an existing calendar you must define the calendar_id: -ci\n";
          exit;
        }
 }


if ($action eq 'c')
 {
   if ($duration_until ne '')
    {
     @dur9 = split (/ /, $duration_until); 
     $dur_date=$dur9[0];
     $dur_time=$dur9[1];

     $ck=$duration_until;
     $vl=&validate();
     if ($vl eq '0')
      {
        print "=> Invalid date/time format \"yyyy-mm-dd hh:mm:ss\" ($duration_until)\n";
        exit 1;
      }

     @durd9 = split (/-/, $dur_date); 
     $durd_yyyy=$durd9[0];
     $durd_mm=$durd9[1];
     $durd_dd=$durd9[2];

     @durt9 = split (/:/, $dur_time); 
     $durt_hour=$durt9[0];
     $durt_min=$durt9[1];
     $durt_sec=$durt9[2];
   }

  if ($duration_from ne '')
   {
     @dur8 = split (/ /, $duration_from); 
     $durf_date=$dur8[0];
     $durf_time=$dur8[1];

     $ck=$duration_from;
     $vl=&validate();
     if ($vl eq '0')
      {
        print "=> Invalid date/time format \"yyyy-mm-dd hh:mm:ss\" ($duration_until)\n";
        exit 1;
      }

     @durf8 = split (/-/, $durf_date); 
     $d_year=$durf8[0];
     $d_month=$durf8[1];
     $d_day=$durf8[2];

     @durf8 = split (/:/, $durf_time); 
     $d_hour=$durf8[0];
     $d_min=$durf8[1];
     $d_sec=$durf8[2];
   }
  $display_from="$d_year-$d_month-$d_day $d_hour:$d_min:$d_sec";

  $s_days=int($duration_seconds/ (24*60*60));
  $s_hours=($duration_seconds/ (60*60) %24);
  $s_minutes=($duration_seconds/60) %60;
  $s_seconds=$duration_seconds%60;
# - if duration more than 1 day, recalculate in hours and add with hours
  if ($s_days > 0)
   {
   $s_daysh=$s_days*24;
   $s_hours=$s_hours+$s_daysh;
   }

  if ($duration_until ne '')
   {
   $s_hours="";
   $s_minutes="";
   $s_seconds="";
   } 
 } 

if ($delete_maint ne 'n')
{
print "\n";
print "Processing filters:\n";
print "-------------------\n";
print "  remove server(s)...: $delete_maint\n"; 
print "  simulate run.......: $simulate\n";
print "\n";
}
if ($serverregex eq 'y')
{
print "\n";
print "Processing filters:\n";
print "-------------------\n";
print "Server selection:\n";
print "  group selected...: $input_group\n"; 
print "  server selected..: $file_list\n";
print "  server regex.....: $file_list_regex\n";
print "  input file.......: $file_path\n";
print "Calendar definitions:\n";
print "  seconds:.........: $duration_seconds\n";
print "  from date:.......: $duration_from\n";
print "  until date.......: $duration_until\n";
print "Hub selection:\n";
print "  name include.....: $s_hub_inc\n";
print "  name exclude.....: $s_hub_exc\n";
print "  origin include...: $s_hub_origin_inc\n";
print "  origin exclude...: $s_hub_origin_exc\n";
print "Device selection:\n";
print "  name include.....: $s_dev_inc\n";
print "  name exclude.....: $s_dev_exc\n";
print "Robot selection:\n";
print "  name include.....: $s_robot_inc\n";
print "  name exclude.....: $s_robot_exc\n";
print "  User1 include....: $s_robot_user1\n"; 
print "  User2 include....: $s_robot_user2\n"; 
print "Generic selection:\n";
print "  bypass hub robots: $bypass_hub\n";
print "  simulate run.....: $simulate\n";
print "\n";
}

}

sub usage
 {
   print "      \n";
   print "Perform UIM maintenance tasks via the rest services:\n";
   print "      \n";
   print " Input parameters: (except for -dm and -ls) (CM_COMPUTER_SYSTEM)\n";
   print "  -if: input path and file with servers to put/remove in maint\n";
   print "  -ih: comma separated list of servers to put/remove in maint (NO regex)\n";
   print "  -ir: regex device pattern to be matched against cm_computer_system table\n";
   print "  -ig: USM group name (last qualifier) to select all servers in that group\n";
   print "       (you can use -ti/-te and -ni/-ne for groups)\n";
   print "\n";
   print " Input Server selection parameters (CM_NIMBUS_ROBOT):\n";
   print " (only when you don't select another input parameter: -if, -ih, -ir or -ig)\n";
   print "  -hi: (opt), hUb name include\n";
   print "  -he: (opt), hUb name exclude\n";
   print "  -ho: (opt), hub origin include\n";
   print "  -hx: (opt), hub origin exclude\n";
   print "  -ni: (opt), device name include\n";
   print "  -ne: (opt), device name exclude\n";
   print "  -ti: (opt), robot name include\n";
   print "  -te: (opt), robot name exclude\n";
   print "  -u1: (opt), robot os_user1 include\n";
   print "  -u2: (opt), robot os_user2 include\n";
   print "  -sim: (opt), simulate regex selection, default: y\n";
   print "\n";
   print " Define calendar and servers:\n";
   print "  -cs: calendar duration in seconds\n";    
   print "  -cf: calendar from (yyyy-mm-dd hh:mm:ss)\n";
   print "  -cu: calendar until (yyyy-mm-dd hh:mm:ss)\n";
   print "  -ct: calendar title (default: server name)\n";
   print "\n";
   print " Add server into existing calendar:\n";
   print "  -ci: input calendar id of existing calendar profile\n";
   print "  -ac: action: \n";
   print "       a: add server to existing calendar specified with -ci\n";
   print "       r: remove server from existing calendar specified with -ci\n";
   print "       c: add calendar and server \n";
   print "       Default: c (when -ih used)\n";
   print "  -db: debug (y,n) default: n\n";
   print "  -bh: bypass hub robots (y,n) default: y\n";
   print "      \n";
   print " List parameters:\n";
   print "  -lc: list existing calendars to obtain calendar id (y,n) Default: n\n";
   print "  -lm: list hosts that are current in maintenance (y,n) Default: n\n";
   print "  -ls: is this server in maintenance? (result=> rc:0 yes rc:1 no)\n";
   print "  -le: list empty calendars (y,n) default: n\n";
   print "  -lx: list expired calendars (y,n) default: n\n";
   print "      \n";
   print " Delete parameters: (only MSsql)\n";
   print "  -dm: delete server regex from maintenance\n";
   print "  -de: delete empty calendars (y,n) default: n\n";
   print "  -dx: delete expired calendars (y,n) default: n\n";
   print "\n";
   print "Examples:\n";
   print " - list calendars: perl nimsoft_rest_maintenance.pl -lc\"y\"\n";
   print " - list servers in maint: perl nimsoft_rest_maintenance.pl -lm\"y\"\n";
   print " - set in maint (into existing calendar):\n";
   print "     perl nimsoft_rest_maintenance.pl -ih\"servera\" -ci\"1\"\n";
   print " - set in maint for 1 hour (and define new calendar at the same time):\n";
   print "     perl nimsoft_rest_maintenance.pl -ih\"servera\" -cs\"3600\"\n";
   print " - check if a server is in maint: nimsoft_rest_maintenance -ls\"servera\"\n";
   print " - remove server from maint: nimsoft_rest_maintenance -dm\"servera\"\n";
   exit;
 }